/*-----------------------------------------------------------------------------

 2002, Jeff M Soft und Hardware GmbH, All Rights Reserved

-----------------------------------------------------------------------------*/

#include "ExtEvtIn.h"
#include "SEMod_struct.h"
#include "SEPin.h"
#include <assert.h>


// define some constants to make referencing in/outs clearer
#define PN_LOWVAL 0
#define PN_HIGHVAL 1
#define PN_CHECKDELAY 2
#define PN_PULSEWIDTH 3
#define PN_MODE 4
#define PN_NAME 5
#define PN_OUTPUT1 6

// The most important part, processing the audio
void Module::sub_process(long buffer_offset, long sampleFrames )
{
	// assign some pointers to your in/output buffers. usually blocks (array) of 96 samples
    float *out1  = buffer_offset + output1_buffer;
    float *lowval = buffer_offset + low_value;
    float *highval = buffer_offset + high_value;
    float *pw = buffer_offset + pulse_width;
    float *cd = buffer_offset + check_delay;

    while(--sampleFrames >= 0) // sampleFrames = how many samples to process (can vary). repeat (loop) that many times
    {
    float p, c;

    // Fix pulse width from 0 to 60 seconds
    p = *pw;
    if(p < 0.0)
        p = 0.0;
    else if(p > 60.0)
        p = 60.0;

    // Fix check delay from 0 to 60 seconds
    c = *cd;
    if(c < 0.0)
        c = 0.0;
    else if(c > 60.0)
        c = 60.0;

    // Determine number of samples for pulse with and check delay
    pw_total = (int)(*pw * rate);
    cd_total = (int)(*cd * rate);

    // Make sure current count is less than total
    if(pw_count > pw_total)
        pw_count = pw_total;

    if(cd_count > cd_total)
        cd_count = cd_total;

    // Check event object if needed
    if(cd_count <= 0)
        {
        DWORD test;

        cd_count = cd_total;
        test = WaitForSingleObject(evt, 0);

        if(test == WAIT_OBJECT_0)
            result = true;
        else
            result = false;
        }
    else
        cd_count--;

    // Process output
    if(mode == 0) // Pulsed
        {
        if(result)
            {
            // In pulsed mode, auto reset the event
            ResetEvent(evt);
            result = false;
            pw_count = pw_total; // Set the pulse count left
            }

        if(pw_count > 0)
            {
            // If any pulse count is left, output is high
            *out1 = *highval;
            pw_count--;
            }
        else
            *out1 = *lowval;

        }
    else // Continuous
        {
        // Output is based on the value of the result
        *out1 = (result ? *highval : *lowval);
        }

    out1++;
    lowval++;
    highval++;
    pw++;
    cd++;
    }
}


//Module::Module(seaudioMasterCallback seaudioMaster) : SEModule_base(seaudioMaster)
Module::Module(seaudioMasterCallback2 seaudioMaster, void *p_resvd1) : SEModule_base(seaudioMaster, p_resvd1)
{
result = false;
pw_count = cd_count = 0;
evt = NULL;
}

void Module::open()
{
	SEModule_base::open();

	// choose which function is used to process audio
	SET_PROCESS_FUNC(Module::sub_process);

    rate = (float)getSampleRate();

	getPin(PN_OUTPUT1)->TransmitStatusChange( SampleClock(), ST_RUN);
}

void Module::close()
{

}

Module::~Module()
{
	// This is where you free any memory/resources your module has created
    if(evt)
        {
        CloseHandle(evt);
        evt = NULL;
        }

}

// describe your module
bool Module::getModuleProperties (SEModuleProperties* properties)
{
	// describe the plugin, this is the name the end-user will see.
	properties->name = "ExtEvent In";

	// return a unique string 32 characters max
	// if posible include manufacturer and plugin identity
	// this is used internally by SE to identify the plug.
	// No two plugs may have the same id.
	properties->id = "BAVII ExtEventIn";

	// Info, may include Author, Web page whatever
#if defined(MINGW) || defined(__GNUC__)
	properties->about = "by Brian Allen Vanderburg II (GNU)";								// !!TODO!!
#elif defined _MSC_VER //hmm don't seem to work
	properties->about = "by Brian Allen Vanderburg II (MS)";
#else
	properties->about = "by  Brian Allen Vanderburg II (Borland?)";
#endif
	return true;
}

// describe the pins (plugs and parameters)
bool Module::getPinProperties (long index, SEPinProperties* properties)
{
	switch( index )								// !!TODO!! list your in / out plugs
	{
    // typical input plug (inputs are listed first)
	// typical output plug
	case PN_LOWVAL:
		properties->name				= "Low Value";
		properties->variable_address	= &low_value;
		properties->direction			= DR_PARAMETER;
		properties->datatype			= DT_FSAMPLE;
        properties->default_value       = "0.0";
		break;
    case PN_HIGHVAL:
		properties->name				= "High Value";
		properties->variable_address	= &high_value;
		properties->direction			= DR_PARAMETER;
		properties->datatype			= DT_FSAMPLE;
        properties->default_value       = "5.0";
		break;
    case PN_CHECKDELAY:
		properties->name				= "Check Delay";
		properties->variable_address	= &check_delay;
		properties->direction			= DR_PARAMETER;
		properties->datatype			= DT_FSAMPLE;
        properties->default_value       = "5.0";
		break;
    case PN_PULSEWIDTH:
		properties->name				= "Pulse Width";
		properties->variable_address	= &pulse_width;
		properties->direction			= DR_PARAMETER;
		properties->datatype			= DT_FSAMPLE;
        properties->default_value       = "1.0";
		break;
    case PN_MODE:
		properties->name				= "Mode";
		properties->variable_address	= &mode;
		properties->direction			= DR_PARAMETER;
		properties->datatype			= DT_ENUM;
        properties->datatype_extra      = "Pulsed=0,Continuous=1";
        properties->default_value       = "0";
		break;
    case PN_NAME:
		properties->name				= "Event Name";
		properties->variable_address	= &name;
		properties->direction			= DR_PARAMETER;
		properties->datatype			= DT_TEXT;
        properties->default_value       = "EVENT";
		break;
    case PN_OUTPUT1:
        properties->name				= "Output";
		properties->variable_address	= &output1_buffer;
		properties->direction			= DR_OUT;
		properties->datatype			= DT_FSAMPLE;
        break;
	default:
		return false; // host will ask for plugs 0,1,2,3 etc. return false to signal when done
	};

	return true;
}

// An input plug has changed value
void Module::OnPlugStateChange(SEPin *pin)
{
    switch(pin->getPinID())
        {
        case PN_MODE:
            pw_count = cd_count = 0;
            result = false;
            ResetEvent(evt);
            break;

        case PN_NAME:
            // Close old event
            if(evt)
                {
                CloseHandle(evt);
                evt = NULL;
                }

            // Get new name
            strcpy(evtname, "SE_EXTEVT_IN_");
            strncat(evtname, name, 128);
            evtname[127] = '\0';

            // Open new event
            evt = CreateEvent(NULL, TRUE, FALSE, evtname);
            break;
        }
}
